import psycopg2

from data.conn.Psycopg2Cursor import Psycopg2Cursor
from data.conn.DatabaseConnection import DatabaseConnection
from data.conn.DatabaseCursor import DatabaseCursor

class Psycopg2Connection(DatabaseConnection):
    def __init__(self, credentials: dict) -> None:


        if 'options' in credentials.keys():
            con = psycopg2.connect(host=credentials['host'], 
            database=credentials['database'],
            user=credentials['user'],
            password=credentials['password'],
            port=credentials['port'],
            options=credentials['options'])
        else:
            con = psycopg2.connect(host=credentials['host'], 
            database=credentials['database'],
            user=credentials['user'],
            password=credentials['password'],
            port=credentials['port'])

        self.conn = con

    def cursor(self) -> DatabaseCursor:
        return Psycopg2Cursor(self.conn.cursor())

    def commit(self) -> None:
        self.conn.commit()

    def rollback(self) -> None:
        self.conn.rollback()



from typing import List, Tuple
from psycopg2.extensions import cursor as Psycopg2CursorType
from psycopg2.extras import execute_batch
from data.conn.DatabaseCursor import DatabaseCursor

class Psycopg2Cursor(DatabaseCursor):
    def __init__(self, cursor: Psycopg2CursorType) -> None:
        self.cursor = cursor

    def execute(self, query: str, params: Tuple = ()) -> None:
        self.cursor.execute(query, params)

    def fetchall(self) -> List[Tuple]:
        return self.cursor.fetchall()

    def fetchone(self) -> Tuple:
        return self.cursor.fetchone()
    
    def execute_batch(self, query: str, lst_data: List[dict]):
        execute_batch(self.cursor, query, lst_data, page_size=100)

    def close(self) -> None:
        self.cursor.close()


from abc import ABC, abstractmethod
from typing import Any, Tuple, List

class DatabaseCursor(ABC):
    @abstractmethod
    def execute(self, query: str, params: Tuple = ()) -> None:
        pass

    @abstractmethod
    def fetchall(self) -> List[Tuple]:
        pass

    @abstractmethod
    def fetchone(self) -> Tuple:
        pass

    @abstractmethod
    def execute_batch(query: str, lst_data: list[dict]): pass

    @abstractmethod
    def close(self) -> None: pass


import abc

class GenericDAO(abc.ABC):

    @abc.abstractmethod
    def find_by_id(self, id: str | int): pass

    @abc.abstractmethod
    def find_by_cond(self, cond: str): pass

    @abc.abstractmethod
    def find_by_data(self, data: dict): pass

    @abc.abstractmethod
    def create(self, data: dict) -> str: pass

    @abc.abstractmethod
    def create_with_batch(self, lst_data: list[dict]) -> str: pass

    @abc.abstractmethod
    def update(self, primary_key_value, data: dict, primary_key_name='id'): pass

    @abc.abstractmethod
    def delete(self, id: str): pass


import abc
from data.conn.DatabaseCursor import DatabaseCursor

class DatabaseConnection(abc.ABC):
    @abc.abstractmethod
    def cursor(self) -> DatabaseCursor:
        pass

    @abc.abstractmethod
    def commit(self) -> None:
        pass

    @abc.abstractmethod
    def rollback(self) -> None:
        pass


from setuptools import setup, find_packages
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='db-dao',
    version='1.3.0',
    author='Gustavo Gomes Dias',
    author_email='gustavo.dias@cedisa.com.br',
    packages=find_packages(),  # Detecta todos os pacotes automaticamente
    long_description=long_description,
    long_description_content_type='text/markdown'
)



from data.DAO.GenericDAO import GenericDAO
from data.conn.DatabaseConnection import DatabaseConnection

class GenericDAOImp(GenericDAO):
    def __init__(self, table: str, conn: DatabaseConnection) -> None:
        self.conn = conn
        self.table = table

    def find_by_id(self, id: str | int):
        query = f'SELECT * FROM {self.table} WHERE {self.table}.id = %s'
        try:
            cur = self.conn.cursor()
            cur.execute(query, (id,))
            data = cur.fetchall()

            return data
        except Exception as e:
            raise e
        

    def find_by_data(self, data: dict):
        field = str(list(data.keys())[0])
        query = f'SELECT * FROM {self.table} WHERE {self.table}.{field} = %s'
        value = data[field]
        try:
            cur = self.conn.cursor()
            cur.execute(query, (value,))
            data = cur.fetchall()

            return data
        except Exception as e:
            raise e

    def find_by_cond(self, cond: str):
        query = f'''SELECT * FROM {self.table} WHERE {cond}'''
        try:
            cur = self.conn.cursor()
            cur.execute(query)
            data = cur.fetchall()

            return data
        except Exception as e:
            raise e
        

    def create(self, data: dict) -> str:

        fields = ', '.join(data.keys())
        sql_values = ', '.join('%s' for _ in data.values())
        query = f"INSERT INTO {self.table} ({fields}) VALUES ({sql_values}) RETURNING id"
    
        values = tuple(data.values())
        try:
            cur = self.conn.cursor()
            cur.execute(query, values)
            self.conn.commit()
            return cur.fetchone()[0]
        except Exception as e:
            self.conn.rollback()
            raise e
        
    def create_with_batch(self, lst_data: list[dict], on_conflict: list[str] = []) -> str:
        if not lst_data:
            return
        
        columns = list(lst_data[0].keys())
        
        placeholders = ', '.join(['${}'.format(i+1) for i in range(len(columns))])
        column_names = ', '.join(columns)
        conflict_column = ', '.join(on_conflict)
        if len(on_conflict) > 0:
            query = 'PREPARE insertStmt AS INSERT INTO {} ({}) VALUES ({}) ON CONFLICT ({}) DO NOTHING'.format(self.table, column_names, placeholders, conflict_column)
        else:
            query = 'PREPARE insertStmt AS INSERT INTO {} ({}) VALUES ({})'.format(self.table, column_names, placeholders)

        cur = self.conn.cursor()
        cur.execute(query)
        cur.execute_batch("EXECUTE insertStmt ({})".format(', '.join('%({})s'.format(col) for col in columns)), lst_data)
        
        cur.execute("DEALLOCATE insertStmt")
        self.conn.commit()
        cur.close()
        
    def update(self, primary_key_value, data: dict, primary_key_name='id'):
        set_fields = ', '.join([f"{key} = %s" for key in data.keys()])
        
        query = f"UPDATE {self.table} SET {set_fields} WHERE {primary_key_name} = %s"
        
        values = list(data.values())
        values.append(primary_key_value)

        try:
            cur = self.conn.cursor()
            cur.execute(query, tuple(values))
            self.conn.commit()
            # return cur.rowcount
        except Exception as e:
            self.conn.rollback()
            raise e
        
    
    def delete(self, where_field: str ,id: str):
        query = f"""DELETE FROM {self.table} WHERE {where_field} = '{id}'"""
        try:
            cur = self.conn.cursor()
            cur.execute(query)
            self.conn.commit()
        except Exception as e:
            self.conn.rollback()
            raise e